using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.NetworkInformation;
using System.Threading;

namespace LibJasinski.Network
{
	public enum ProxyState { NotUsing, Using, Verified }
	public delegate void NoArgsDelegate();
	public class InternetConnector
	{
		private static InternetConnector _default;

		static InternetConnector()
		{
			_default = new InternetConnector();
		}
		internal static InternetConnector Default
		{
			get { return _default; }
		}


		public event NoArgsDelegate ProxyStateChanged;
		private string _appName;
		private string _proxyServer, _proxyUser, _proxyUserDomain, _proxyPassword;
		private ProxyState _proxyState;

		public InternetConnector()
		{
			Constructor("");
		}
		public InternetConnector(string appName)
		{
			Constructor(appName);
		}
		public WebClient GetConnection()
		{
			WebClient result = new WebClient();
			bool canPing = true;

			for (int i = 0; i < 3; i++)
			{
				canPing = CanPing(Cfg.Def.ProxyServer);
				if (canPing) break;
				Thread.Sleep(3000);
			}

			if (!canPing)
			{
				this.ProxyState = ProxyState.NotUsing;
			}
			else if (InitProxy())
			{
				NetworkCredential cred = new NetworkCredential(_proxyUser, _proxyPassword, 
					_proxyUserDomain);

				result.Proxy = new WebProxy(_proxyServer, true, null, cred);

				this.ProxyState = ProxyState.Using;
			}

			return result;
		}

		private bool InitProxy()
		{
			bool result = false;

			if (this.ProxyState == ProxyState.Verified)
			{
				result = true;
			}
			else
			{
				_proxyServer = Cfg.Def.ProxyServer;
				_proxyUser = Cfg.Def.ProxyUser;
				_proxyUserDomain = Cfg.Def.ProxyUserDomain;

				LoginForm f = new LoginForm();
				result = (f.Login(this));
				f.Dispose();
				f = null;
			}
			return result;
		}
		internal string AppName
		{
		    get { return _appName; }
			set { _appName = value; }
		}
		public string ProxyServer { 
			get { return _proxyServer; }
			internal set { _proxyServer = value; }
		}
		public ProxyState ProxyState
		{
		    get { return _proxyState; }
		    set 
			{
				bool changed = (_proxyState != value);
				_proxyState = value; 
				if (changed && this.ProxyStateChanged != null)
					this.ProxyStateChanged();
			}
		}
		internal string ProxyUser { 
			get { return _proxyUser; }
			set { _proxyUser = value; }
		}
		internal string ProxyUserDomain { 
			get { return _proxyUserDomain; }
			set { _proxyUserDomain = value; }
		}
		internal string ProxyPassword
		{
			set { _proxyPassword = value; }
		}

		private void Constructor(string appName)
		{
			this.ProxyState = ProxyState.NotUsing;
			_appName = appName;
		}
		private static bool CanPing(string address)
		{
			bool result = false;
			try 
			{
				if (address.Length > 0 && new Ping().Send(address).Status == IPStatus.Success)
					result = true;
			}
			catch {}
			return result;
		}

		//public void ResetProxy()
		//{
		//    _proxyState = ProxyState.NotUsing;
		//}

	}
}
